	org 32768


cursor_colour	equ 0x78	;bright white paper, black ink
bg_colour	equ 7	;dark white ink on black paper

;set up interrupts
interrupt_init

	ld hl,0xBE00

	ld de,0xBE01

	ld (hl),0xBF
	ld b,e
	ld c,e	; ld bc,257

	ldir

	ld hl,0xBFBF
	ld (hl),0xC3	; opcode for JP

	; set up screen
	xor a
	out (254),a		;black border
	; set system variables for nice printing
	ld (23659),a	;DF_SZ - use all 24 lines for upper screen
	ld (23612),a	;TVFLAG - print to upper screen
	cpl
	ld (23696),a	;MASK_T - all attributes transparent

	ld hl,16384
	ld de,16385
	ld (hl),l
	ld bc,6144
	ldir			; clear screen
	ld (hl),bg_colour
	ld bc,767
	ldir			; cover whole screen with bg_colour

	;fill in the numbers
	ld hl,65536-10
	; we will loop from 0 to 23 (b is 0 now)
allnum_loop
	push bc
	push hl
	call put_numbers
	pop hl
	inc hl
	pop bc
	inc b
	ld a,b
	cp 23
	jr c,allnum_loop

;draw cursor
	xor a			; set cursor to start at first column
	call set_cursor
	ld a,cursor_colour
	ld (hl),a

;wait for a key
wait_key
	call 0x028e	;keyscan
	jr nz,wait_key		; loop if too many keys pressed
	ld a,d
	inc a
	jr nz,wait_key		; non-0xff in D means two keys pressed at once
	ld a,e
	cp 0x1a		;keycode for O
	jp z,key_o
	cp 0x22		;keycode for P
	jp z,key_p
	cp 0x21		;keycode for enter
	jp z,key_replay
	cp 0x20		;keycode for space
	jp z,key_replay
	cp 0x25
	jp z,key_q	;keycode for Q
	cp 0x27
	jp z,key_cs	;keycode for caps shift

	; check all keys corresponding to hex digits
	ld b,0
	cp 0x23		;code for 0
	jp z,key_digit
	inc b
	cp 0x24		;code for 1
	jp z,key_digit
	inc b
	cp 0x1c		;code for 2
	jp z,key_digit
	inc b
	cp 0x14		;code for 3
	jp z,key_digit
	inc b
	cp 0x0c		;code for 4
	jp z,key_digit
	inc b
	cp 0x04		;code for 5
	jp z,key_digit
	inc b
	cp 0x03		;code for 6
	jp z,key_digit
	inc b
	cp 0x0b		;code for 7
	jp z,key_digit
	inc b
	cp 0x13		;code for 8
	jp z,key_digit
	inc b
	cp 0x1b		;code for 9
	jp z,key_digit
	inc b
	cp 0x26		;code for A
	jp z,key_digit
	inc b
	cp 0x00		;code for B
	jp z,key_digit
	inc b
	cp 0x0f		;code for C
	jp z,key_digit
	inc b
	cp 0x16		;code for D
	jp z,key_digit
	inc b
	cp 0x15		;code for E
	jp z,key_digit
	inc b
	cp 0x0e		;code for F
	jp z,key_digit
	cp 0x0d		;code for R
	jp z,key_from_here
	cp 0x0a		;code for U
	jp z,key_from_start
	jp wait_key

key_o
	ld hl,(cursor_screenx)
	ld (hl),bg_colour	;clear cursor
	ld a,(cursor_x)
	or a
	jr z,no_move_cursor	;don't move cursor past the left hand side
	dec a
no_move_cursor
	call set_cursor
	ld (hl),cursor_colour
	jr debounce_and_return

key_p
	ld hl,(cursor_screenx)
	ld (hl),bg_colour		; clear cursor
	ld a,(cursor_x)
	cp 13
	jr z,no_move_cursor	;don't move cursor past the right hand side
	inc a
	jr no_move_cursor

key_q
;scroll back a row (ie scroll screen down)
	;shift bottom third down
	ld hl,0x57bf	;last byte of 22nd row
	ld de,0x57df	;last byte of 23rd row
	ld bc,0x07c0
	lddr
	;move bottom line of middle third to top of bottom third
	ld hl,0x48e0
	ld de,0x5000
	call copy_row
	;shift middle third down
	ld hl,0x4fdf
	ld de,0x4fff
	ld bc,0x07e0
	lddr
	;move bottom line of top third to top of middle third
	ld hl,0x40e0
	ld de,0x4800
	call copy_row
	;shift top third down
	ld hl,0x47df
	ld de,0x47ff
	ld bc,0x07e0
	lddr
	call clear_bottom_line
	;draw new top line
	ld hl,(cursor_y)
	ld de,0xfff5	;equivalent to -11
	add hl,de		;find the row number 11 places above the cursor
	ld b,0
	call put_numbers

	; update cursor position
	ld hl,(cursor_y)
	dec hl
	ld (cursor_y),hl
	ld hl,(cursor_memy)
	ld de,0xfff9	;equivalent to -7
	add hl,de
	ld (cursor_memy),hl
debounce_and_return
	call key_debounce
	jp wait_key

key_cs
;scroll forward a row (ie scroll screen up)
	;shift top third up
	ld hl,0x4020
	ld de,0x4000
	ld bc,0x07e0
	ldir
	;copy top line of middle third to bottom of top third
	ld hl,0x4800
	ld de,0x40e0
	call copy_row
	;shift middle third up
	ld hl,0x4820
	ld de,0x4800
	ld bc,0x07e0
	ldir
	;copy top line of bottom third to bottom of middle third
	ld hl,0x5000
	ld de,0x48e0
	call copy_row
	;shify bottom third up
	ld hl,0x5020
	ld de,0x5000
	ld bc,0x07c0
	ldir
	call clear_bottom_line
	;draw new bottom line
	ld hl,(cursor_y)
	ld de,13
	add hl,de	;find the row number 13 places below the cursor
	ld b,22
	call put_numbers

	;update cursor position
	ld hl,(cursor_y)
	inc hl
	ld (cursor_y),hl
	ld hl,(cursor_memy)
	ld de,7
	add hl,de
	ld (cursor_memy),hl
	jr debounce_and_return

key_from_start
	ld hl,dat
join1
	call play_from_point
	jr debounce_and_return

key_from_here
	ld hl,(cursor_memy)
	jr join1

key_digit
	;enter with digit in low-nibble of B
	ld a,(cursor_x)		;low bit of cursor_x set indicates
	srl a				;we're on the low nibble
	jr c,set_low_nibble
;set high nibble
	ld de,(cursor_memy)	;find address of the byte under the cursor
	ld h,0
	ld l,a
	add hl,de

	ld a,(hl)
	and 0x0f	;clear old highnibble
	sla b
	sla b
	sla b
	sla b
	or b		;replace with new one
	ld (hl),a
	call refresh_cursor_line
key_replay
	call trynote
	jp wait_key

set_low_nibble
	ld de,(cursor_memy)
	ld h,0
	ld l,a
	add hl,de
	ld a,(hl)
	and 0xf0	;clear old lownibble
	or b		;replace with new one
	ld (hl),a
	call refresh_cursor_line
	jr key_replay

; exit ;(no need)
; 	ld hl,23659	; make DF_SZ sensible again
; 	ld (hl),2

;	ret

key_debounce
; provide a short delay to give chance for user to release the key
	ld b,8		;160ms seems about right
key_debounce_loop
	halt
	push bc
	ld bc,0x00fe	;scan all keys
	in a,(c)
	pop bc
	cpl
	and %00011111	;zero = no keys pressed
	ret z
	djnz key_debounce_loop
	ret

copy_row
;copy a screen line from hl to de
	ld b,8
copy_row_lp
	push hl
	push de
	push bc
	ld bc,32
	ldir
	pop bc
	pop de
	pop hl
	inc d
	inc h
	djnz copy_row_lp
	ret

clear_bottom_line
; do something profoundly baffling.
	ld hl,0x50e0
	push hl
	ld de,0x50e1
	ld bc,31
	ld (hl),0
	ldir
	pop hl
	ld de,0x51e0
	ld b,7
scrl_up_3
	push hl
	push de
	push bc
	ld bc,32
	ldir
	pop bc
	pop de
	pop hl
	inc d
	djnz scrl_up_3
	ret

set_cursor
; given cursor x [0..13] in a, set variables cursor_x and cursor_screenx
; and return cursor_screen in hl
	ld (cursor_x),a
	ld e,a
	ld d,0
	ld hl,cursor_map	; see which screen position this corresponds to
	add hl,de
	ld l,(hl)
	ld h,0
	ld de,22528+320	;start of cursor line
	add hl,de
	ld (cursor_screenx),hl
	ret

cursor_x	db 0
cursor_screenx	dw 0
cursor_y	dw 0
cursor_memy	dw dat

cursor_map	db 5,6, 8,9, 11,12, 14,15, 17,18, 20,21, 23,24


; rewrite the line under the cursor
refresh_cursor_line
	ld hl,(cursor_y)
	ld b,10
;now continue into put_numbers

; write a row of numbers
; enter with: hl = row number, b = screen line
put_numbers
	ld a,0x16	;first, set appropriate PRINT AT position
	rst 0x10
	ld a,b
	rst 0x10
	xor a
	rst 0x10
	bit 7,h		; if hl is 'negative', draw a blank instead
	jr nz,put_blank

	; put row number (number presently in HL) first
	ex de,hl
	ld b,d
	call hex
	ld a,h
	rst 0x10
	ld a,l
	rst 0x10
	ld b,e
	call hex
	ld a,h
	rst 0x10
	ld a,l
	rst 0x10
	ld a,"|"
	rst 0x10
	; now translate row number to an address
	ld h,d
	ld l,e
	add hl,hl	;multiply by 7
	add hl,hl
	add hl,hl
	or a		;reset carry
	sbc hl,de
	ld de,dat
	add hl,de
	; print hex codes for the seven bytes of the row
	ld b,7
hex_chan_lp
	push bc
	push hl
	ld b,(hl)
	call hex
	ld a,h
	rst 0x10
	ld a,l
	rst 0x10
	pop hl
	inc hl
	pop bc
	ld a,b
	dec a
	ret z	; for the last iteration, don't print a trailing symbol
	dec a
	jr nz,no_dot
	ld a,"|"
	rst 0x10
	djnz hex_chan_lp
no_dot
	ld a,"."
	rst 0x10
	djnz hex_chan_lp
		; should never get here

put_blank
	ld b,27
	ld hl,blank_text
put_blank_1
	ld a,(hl)
	inc hl
	rst 0x10
	djnz put_blank_1
	ret

; hex: input byte in B, return as ascii hex in HL
hex
	ld a,b
	srl a
	srl a
	srl a
	srl a
	or 0x30
	cp 0x3a
	jr c,isdigit
	add a,7
isdigit	ld h,a
	ld a,b
	and 0x0f
	or 0x30
	cp 0x3a
	jr c,isdigit2
	add a,7
isdigit2	ld l,a
	ret

blank_text db "    |  .  .  .  .  .  |    "
blank_text_end

; player routines

trynote
	ld hl,playkey
	ld (0xBFC0),hl

	ld hl,(cursor_memy)
	ld a,(hl)
	ld (bchan+1),a
	inc hl
	ld a,(hl)
	ld (cchan+1),a
	inc hl
	ld a,(hl)
	ld (dchan+1),a
	inc hl
	ld a,(hl)
	ld (echan+1),a
	inc hl
	ld a,(hl)
	ld (hchan+1),a
	inc hl
	ld a,(hl)
	ld (lchan+1),a
	jr init_play_common

play_from_point
	ld (datpos+1),hl
	ld hl,playfull
	ld (0xBFC0),hl

init_play_common

	ld hl,lp1
	ld (lpend+1),hl


	di
	ld a,0xBE
	ld i,a
	im 2
	ld a,1		; so that we'll immediately fetch a new note
	ex af,af'
	ei

	ld bc,65533
	ld de,0xBFFF

	ld a,7
	ld b,e
	out (c),a
	ld a,63
	ld b,d
	out (c),a

	ld l,15
	exx
lp1	dec b
	call z,bchan
	dec c
	call z,cchan
	dec d
	call z,dchan
	dec e
	call z,echan
	dec h
	call z,hchan
	dec l
	call z,lchan
;	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
;	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
;	.db 0,0,0,0,0,0,0,0
;	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

lpend
	jp lp1

bchan	ld b,0
	exx
	ld a,8
	ld b,e
	out (c),a
	ld a,8
	ld b,d
	jr acta
cchan	ld c,0
	exx
	ld a,8
	ld b,e
	out (c),a
	ld a,7
	ld b,d
	jr acta

acta	
reca	xor 0
	ld (reca+1),a
	out (c),a
	exx
	ret

dchan	ld d,0
	exx
	ld a,9
	ld b,e
	out (c),a
	ld a,8
	ld b,d
	jr actb
echan	ld e,0
	exx
	ld a,9
	ld b,e
	out (c),a
	ld a,7
	ld b,d
	jr actb

actb
recb	xor 0
	ld (recb+1),a
	out (c),a
	exx
	ret

hchan	ld h,0
	exx
	ld a,10
	ld b,e
	out (c),a
	ld a,8
	ld b,d
	jr actc
lchan	ld l,0
	exx
	ld a,10
	ld b,e
	out (c),a
	ld a,7
	ld b,d
	jr actc

actc
recc	
	xor 0
	ld (recc+1),a
	out (c),a
	exx
	ret

niceexit
	di
	ld a,0x3F
	ld i,a
	im 1
	ei
	ret

playkey
	push bc
	ex af,af'
	ld bc,0x00FE	;scan all keys at once
	in a,(c)
	pop bc
	cpl
	and %00011111	;zero = no keys pressed
	jr nz,noexit
	ld hl,niceexit
	ld (lpend+1),hl
noexit
	ex af,af'
	ei
	ret

playfull
	ex af,af'
	dec a
	jr nz,nana
	push hl
datpos	ld hl,dat

	ld a,(hl)
	ld (bchan+1),a
	inc hl

	ld a,(hl)
	ld (cchan+1),a
	inc hl

	ld a,(hl)
	ld (dchan+1),a
	inc hl

	ld a,(hl)
	ld (echan+1),a
	inc hl

	ld a,(hl)
	ld (hchan+1),a
	inc hl

	ld a,(hl)
	ld (lchan+1),a
	inc hl

	ld a,(hl)
	or a
	jr z,fin
	inc hl
	ld (datpos+1),hl
	pop hl
nana
	push bc
	push af
	ld bc,0x7ffe
	in a,(c)
	and 1
	jr nz,no_exit_fullplay
	ld hl,niceexit
	ld (lpend+1),hl
no_exit_fullplay
	pop af
	pop bc
	ex af,af'
	ei
	ret
fin	pop hl
	pop hl
	ld hl,niceexit
	push hl
	ei
	ret

intend	equ $

dat	db 59,119,79,44,30,39,50
	db 59,119,79,47,30,39,25
	db 59,119,79,53,30,39,25
	db 59,119,79,47,30,39,100
;	db 0,0,0,0,0,0,0
	end
